package at.ac.uibk.gitsearch.service.dto;

import at.ac.uibk.gitsearch.repository.search.SearchRepositoryConstants;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;
import org.elasticsearch.search.aggregations.Aggregations;
import org.elasticsearch.search.aggregations.bucket.terms.Terms;

public class GitFilesAggregationDTO {

    private List<FrequencyDTO<String>> fileFormat;
    private List<FrequencyDTO<String>> repositories;
    private List<FrequencyDTO<String>> university;

    @SuppressWarnings("unchecked")
    public GitFilesAggregationDTO(Aggregations aggregations) {
        Terms terms = aggregations.get(SearchRepositoryConstants.FULLTEXT_SUB_GROUP_AGG);

        Collection<Terms.Bucket> buckets = (Collection<Terms.Bucket>) terms.getBuckets();
        this.university =
            buckets.stream().map(bucket -> new FrequencyDTO<>(bucket.getKeyAsString(), bucket.getDocCount())).collect(Collectors.toList());

        terms = aggregations.get(SearchRepositoryConstants.FULLTEXT_REPOSITORY_AGG);
        buckets = (Collection<Terms.Bucket>) terms.getBuckets();
        this.repositories =
            buckets.stream().map(bucket -> new FrequencyDTO<>(bucket.getKeyAsString(), bucket.getDocCount())).collect(Collectors.toList());

        terms = aggregations.get(SearchRepositoryConstants.FULLTEXT_FILE_EXTENSION_AGG);
        buckets = (Collection<Terms.Bucket>) terms.getBuckets();
        this.fileFormat =
            buckets.stream().map(bucket -> new FrequencyDTO<>(bucket.getKeyAsString(), bucket.getDocCount())).collect(Collectors.toList());
    }

    public List<FrequencyDTO<String>> getFileFormat() {
        return fileFormat;
    }

    public void setFileFormat(List<FrequencyDTO<String>> fileFormat) {
        this.fileFormat = fileFormat;
    }

    public List<FrequencyDTO<String>> getRepositories() {
        return repositories;
    }

    public void setRepositories(List<FrequencyDTO<String>> repositories) {
        this.repositories = repositories;
    }

    public List<FrequencyDTO<String>> getUniversity() {
        return university;
    }

    public void setUniversity(List<FrequencyDTO<String>> university) {
        this.university = university;
    }
}
