import { Person } from 'app/shared/model/person.model';
import { SearchResultDTO } from 'app/shared/model/search/search-result-dto.model';

export enum ILanguages {
  GERMAN = 'de',
  ENGLISH = 'en',
}

export enum IInteractivityType {
  ACTIVE = 'active',
  EXPOSITIVE = 'expositive',
  MIXED = 'mixed',
}

// just a cheap trick, to make enum available to HTML.
// see https://stackoverflow.com/questions/35835984/how-to-use-a-typescript-enum-value-in-an-angular2-ngswitch-statement
// export function IExerciseTypeAware(constructor: Function) {
//    constructor.prototype.IExerciseType = IExerciseType;
// }

// export class IExerciseTypeClass {
//     myEnum: typeof IExerciseType;
//     myEnumField: IExerciseType;
// }

/** @deprecated unnecessary wrapper around exercises. Only causes extra effort. On long term should be replaced by SearchResultDTO  */
export interface Exercise {
  // from user provided metadata
  assesses: string[];
  audience: string;
  contributor: Person[];
  creator: Person[];
  description: string;
  difficulty?: string;
  educationalAlignment: string;
  educationalFramework: string;
  educationalLevel: string[];
  educationalUse: string[];
  format: string[];
  identifier: string;
  interactivityType: IInteractivityType;
  isBasedOn: string[];
  imageURL: string;
  keyword: string[];
  language: string[];
  learningResourceType: string;
  license: string;
  metadataVersion: string;
  programmingLanguage: string[];
  publisher: Person[];
  requires: string[];
  status: string;
  structure: string;
  subject: string[];
  teaches: string[];
  timeRequired: string;
  title: string;
  typicalAgeRange: string;
  version: string;

  gitlabURL: string;

  // not in metadata
  rating: number;
  lastUpdate: Date;
  originalResult: SearchResultDTO;

  // thesis
  views: number;
  downloads: number;
  numberOfLikes: number;
  userHasLiked: boolean;
  numberOfWatchlistEntries: number;
}

export function searchResultToExercise(searchResult: SearchResultDTO): Exercise {
  return {
    assesses: searchResult.metadata.assesses,
    audience: searchResult.metadata.audience,
    contributor: searchResult.metadata.contributor,
    creator: searchResult.metadata.creator,
    description: searchResult.metadata.description,
    difficulty: searchResult.metadata.difficulty,
    educationalAlignment: searchResult.metadata.educationalAlignment,
    educationalFramework: searchResult.metadata.educationalFramework,
    educationalLevel: searchResult.metadata.educationalLevel,
    educationalUse: searchResult.metadata.educationalUse,
    format: searchResult.metadata.format,
    identifier: searchResult.metadata.identifier,
    imageURL: searchResult.metadata.image,
    interactivityType: searchResult.metadata.interactivityType,
    isBasedOn: searchResult.metadata.isBasedOn,
    keyword: searchResult.metadata.keyword,
    language: searchResult.metadata.language,
    learningResourceType: searchResult.metadata.learningResourceType,
    license: searchResult.metadata.license,
    metadataVersion: searchResult.metadata.metadataVersion,
    programmingLanguage: searchResult.metadata.programmingLanguage,
    publisher: searchResult.metadata.publisher,
    requires: searchResult.metadata.requires,
    status: searchResult.metadata.status,
    structure: searchResult.metadata.structure,
    subject: searchResult.metadata.subject,
    teaches: searchResult.metadata.teaches,
    timeRequired: searchResult.metadata.timeRequired,
    title: searchResult.metadata.title,
    typicalAgeRange: searchResult.metadata.typicalAgeRange,
    version: searchResult.metadata.version,

    rating: searchResult.ranking5,
    lastUpdate: searchResult.project.last_activity_at,
    gitlabURL: searchResult.project.url,
    originalResult: searchResult,
    views: searchResult.views,
    downloads: searchResult.downloads,

    numberOfLikes: 0,
    userHasLiked: false,
    numberOfWatchlistEntries: 0,
  };
}
